@@ -588,6 +588,8 @@ GRN_API grn_obj *grn_type_create(grn_ctx *ctx, const char *name, unsigned name_s
 
 GRN_API grn_rc grn_plugin_register(grn_ctx *ctx, const char *name);
 GRN_API grn_rc grn_plugin_register_by_path(grn_ctx *ctx, const char *path);
+GRN_API const char *grn_plugin_get_system_plugins_dir(void);
+GRN_API const char *grn_plugin_get_suffix(void);
 
 /**
  * grn_proc_create:
@@ -937,7 +939,7 @@ typedef struct {
 /**
  * grn_index_cursor_open
  * @tc: 対象cursor
- * @index: 対象cursor
+ * @index: 対象インデックスカラム
  * @rid_min: 出力するレコードidの下限
  * @rid_max: 出力するレコードidの上限
  *
@@ -1340,6 +1342,16 @@ GRN_API grn_obj *grn_obj_get_element_info(grn_ctx *ctx, grn_obj *obj, grn_id id,
 GRN_API grn_rc grn_obj_set_element_info(grn_ctx *ctx, grn_obj *obj, grn_id id,
                                         grn_info_type type, grn_obj *value);
 
+/**
+ * c:function:: grn_bool grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj)
+ *
+ * Check whether groonga built-in object.
+ *
+ * :param ctx: context
+ * :param obj: target object
+ * :retype: ``GRN_TRUE`` for built-in groonga object, ``GRN_FALSE`` otherwise.
+ **/
+GRN_API grn_bool grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj);
 
 /**
  * grn_obj_get_value:
