@@ -549,6 +549,9 @@ grn_ctx_init(grn_ctx *ctx, int flags)
   grn_gctx.next->prev = ctx;
   grn_gctx.next = ctx;
   CRITICAL_SECTION_LEAVE(grn_glock);
+  ctx->errline = 0;
+  ctx->errfile = "";
+  ctx->errfunc = "";
   ctx->trace[0] = NULL;
   ctx->errbuf[0] = '\0';
   return ctx->rc;
@@ -2388,9 +2391,8 @@ grn_ctx_log(grn_ctx *ctx, char *fmt, ...)
 {
   va_list argp;
   va_start(argp, fmt);
-  vsnprintf(ctx->errbuf, GRN_CTX_MSGSIZE - 1, fmt, argp);
+  vsnprintf(ctx->errbuf, GRN_CTX_MSGSIZE, fmt, argp);
   va_end(argp);
-  ctx->errbuf[GRN_CTX_MSGSIZE - 1] = '\0';
 }
 
 void
