@@ -209,8 +209,8 @@ struct _grn_ctx_impl {
   grn_edge *edge;
   grn_loader loader;
 
-  /* module portion */
-  const char *module_path;
+  /* plugin portion */
+  const char *plugin_path;
 
   /* output portion */
   grn_content_type output_type;
@@ -224,6 +224,9 @@ struct _grn_ctx_impl {
   /* match escalation portion */
   int64_t match_escalation_threshold;
 
+  /* lifetime portion */
+  grn_proc_func *finalizer;
+
   /* ql portion */
   uint32_t ncells;
   uint32_t seqno;
